@@ -42,6 +42,7 @@ def model_dict_to_str(
     set_links_d: T.Dict[str, "Resolver"] = None,
     force_replace: bool = False,  # make this true for inserts
     set_strs: T.List[str] = None,
+    error: bool = False,
 ) -> T.Tuple[str, T.Dict[str, T.Any]]:
     from .resolver import UpdateOperation
 
@@ -50,11 +51,21 @@ def model_dict_to_str(
     for field_name, val in d.items():
         add_to_variables: bool = True
         type_cast_str = edgedb_conversion_map.get(field_name, {}).get("cast")
+        if val is None:
+            val = set()
+
         if not type_cast_str:
             # resolvers will come up here so ignore them
             if (set_links_d and field_name in set_links_d) or val == set():
                 continue
-            raise NodeException(f"{field_name=} not in {edgedb_conversion_map=}")
+            # if this is a resolver, it is fine... also what if it is none?
+            if error:
+                raise NodeException(
+                    f"{field_name=}, {val=} not in {edgedb_conversion_map=}"
+                )
+            else:
+                continue
+
         variable_name = f"{field_name}{random_str(10)}"
         field_str = f"{field_name} := <{type_cast_str}>${variable_name}"
         if type_cast_str.endswith("::str") and (
@@ -124,7 +135,9 @@ def basemodel_to_str(
     force_replace: bool = False,
 ) -> T.Tuple[str, T.Dict[str, T.Any]]:
     # should exclude_none = True?
-    model_d = model.dict(exclude_none=True, exclude=fields_to_exclude)
+    # model_d = model.dict(exclude_none=True, exclude=fields_to_exclude)
+    # TODO is this right?
+    model_d = model.dict(exclude=fields_to_exclude)
 
     set_links_d = make_set_links_d(
         model=model, model_d=model_d, default_replace=default_replace
@@ -364,10 +377,10 @@ class Node(BaseModel, T.Generic[InsertType, PatchType]):
             conflict_str = custom_on_conflict_str
         elif upsert_given_conflict_on:
             # replace s and variables
-            fields_to_exclude = (
-                set(insert.dict(exclude_none=True).keys())
-                - cls.GraphORM.updatable_fields
-            )
+            # insert_dict = insert.dict(exclude_none=True)
+            # TODO again for exclude_none is this right?
+            insert_dict = insert.dict()
+            fields_to_exclude = set(insert_dict.keys()) - cls.GraphORM.updatable_fields
             update_s, update_variables = basemodel_to_str(
                 insert, default_replace=True, fields_to_exclude=fields_to_exclude
             )
@@ -414,7 +427,7 @@ class Node(BaseModel, T.Generic[InsertType, PatchType]):
             custom_on_conflict_str=custom_on_conflict_str,
         )
         if not inserts:
-            return set()
+            return list()
         insert_str = creating_strings.insert_str_from_cls(
             insert_cls=inserts[0].__class__,
             node_cls=cls,
@@ -431,7 +444,8 @@ for item in json_array_unpack(raw_data) union ({insert_str}) {given_resolver.to_
         print(f"{full_insert_str=}")
         data: T.List[T.Dict[str, T.Any]] = []
         for insert in inserts:
-            d = insert.dict(exclude_none=True)
+            # d = insert.dict(exclude_none=True)
+            d = insert.dict()
             # if the cast says they should be strings and they are not, turn them into json
             for field_name, value in d.items():
                 if cast := cls._edgedb_conversion_map.get(field_name, {}).get("cast"):
